#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
import types
import logging
import traceback
import threading

from threading import Event, Thread


DEFAULT_TIMEOUT = 5
"""
Constante que indica el tiempo por defecto tras el cual cada Collector 
recolecta datos
"""

class InitializationError (Exception):
    pass


class CollectorManager:
    """
    <<Singleton>> Clase encargada de lanzar los distintos recolectores que componen el
    sistema.

    Contiene un método para iniciar todos los recolectores y otro para parar la
    ejecución.
    """

    instance = None


    def get_instance(data_manager = None, collectors = []):
        """ Obtiene la instancia del CollectorManager de la aplicación

        @type data_manager: L{DataManager}
        @param data_manager: Instancia del DataManager de la aplicación

        @type collectors: list
        @param collectors: Lista de recolectores que se quieren administrar.
        Estos recolectores deben derivar de la clase Collector.
        """
        if not CollectorManager.instance:
            CollectorManager.instance = CollectorManager(data_manager,
                    collectors)

        return CollectorManager.instance
            
    get_instance = staticmethod(get_instance)


    def __init__(self, data_manager, collectors = []):
        """
        @type collectors: list
        @param collectors: lista con objetos Collector
        """
        self.data_manager = data_manager
        self.collectors = collectors


    def init_collectors(self):
        """Inicializa los recolectores
        """
        for ele in self.collectors:
            ele.start()


    def stop_collectors(self):
        """ Finaliza todos los recolectores
        """
        for ele in self.collectors:
            logging.getLogger().debug("Stopping " + str(ele))
            ele.event.set()
            #ele.stop()

    def wait(self):
        """ Espera a que terminen todos los recolectores
        """
        for ele in self.collectors:
            ele.join()


class LockPool(object):
    ''' Pool de objetos L{threading.Lock} comunes a toda la aplicación.
    '''
    
    lock_dict = {}

    def get_lock(key):
        '''        
        @param key: nombre del objeto Lock
        
        @return: Objeto asociado a la clave indicada. Si el objeto no existe
        en el Pool se crea antes de devolverlo.
        '''
        
        lock_dict = LockPool.lock_dict

        if not lock_dict.has_key(key):
            lock_dict[key] = threading.Lock()

        return lock_dict[key]
    get_lock = staticmethod(get_lock)


class Collector(Thread):
    """
    Clase abstracta que ayuda a la hora de crear recolectores de datos.
    """
    def __init__(self, data_manager, 
            event = Event(), 
            timeout = DEFAULT_TIMEOUT):

        Thread.__init__(self)
        
        # Attributes
        self.event = event
        self.timeout = timeout 
        self.counter = 0
        self.data_manager = data_manager
        self.logger = logging.getLogger()


    def run(self):
        self.collect()
        while not self.event.isSet():
            self.counter += 1
            if self.counter >= self.timeout:
                self.collect()
                self.counter = 0
            time.sleep(1)

    
    def stop(self):
        self.event.set()


    def collect(self):
        raise NotImplementedError


