# -*- coding: utf-8 -*-
import os.path
import logging

from sets     import Set
from datetime import datetime
from copy     import deepcopy
from common.utils import PersistentDict

""" Clases que encapsulan el acceso a los datos de los envíos. """

class FileMetaData(object):

    """ Metadatos de un archivo, sin incluir el propio contenido del archivo. """

    def __init__ (self, basepath, relpath):
        """ 
        @type basepath: str
        @param basepath: Ruta base del fichero de configuración (p.ej.
        /var/vaguada/info)

        @type relpath: str
        @param relpath: Ruta relativa del fichero (relativa a basepath)
        """
        self.__path = basepath + os.sep + relpath

        self.__directory = basepath
        self.__filename = relpath

        self.path = (basepath, relpath)

        aux_basepath = os.sep.join(basepath.split(os.sep)[:-1])
        checksum_file_path = aux_basepath + os.sep + 'metadata' + os.sep +\
                            'checksum_file'

        try:
            self.__checksum =  ChecksumCache(checksum_file_path)[self.ident]
        except:
            self.__checksum = ''
         
        self.__modif_date = None
        self.__corrupted = None

    
    def __get_path(self):
        return self.__directory + os.sep + self.__filename

    def __set_path(self, (basepath, relpath)):
        # base_path = path.rsplit(os.sep, 3)[0]
        aux_basepath = os.sep.join(basepath.split(os.sep)[:-1])
        mod_file_path = aux_basepath + os.sep + 'metadata' + os.sep + 'modification_flag_file'
        self.mod_file_path = mod_file_path

        self.__modified = self.ident in ModCache(mod_file_path)

    path = property(__get_path, __set_path,
            "Path del fichero")

    def __get_directory(self):
        return self.__directory
    directory = property (__get_directory, None, None, 'Directorio')

    def __get_filename(self):
        return self.__filename
    filename = property (__get_filename, None, None, 'Nombre de archivo')

    def __get_checksum(self):
        return self.__checksum
    checksum = property (__get_checksum, None,  None, 'Suma de comprobación')

    def __get_modified(self):
        return self.__modified

    def __set_modified(self, value):
        self.__modified = value
        mc = ModCache(self.mod_file_path)
        if value == True:
            mc.add(self.ident)
        else:
            try:
                mc.remove(self.ident)
            except:
                pass

    modified = property (__get_modified, __set_modified, None, 'El archivo fue modificado')

    def __get_modif_date(self):
        return self.__modif_date
    modif_date = property (__get_modif_date, None, None, 'Fecha de la última modificación')

    def __get_corrupted(self):
        return self.__corrupted
    corrupted = property(__get_corrupted, None, None, 
        'El fichero está corrupto (md5 incorrecto)')


    def __get_ident(self):
        #return self.__directory + os.sep + self.__filename
        return self.__filename

    ident = property(__get_ident, None, None,
            'Identificador del fichero (directorio/nombre). Ejemplo:\
            data/20060101.sessions')


    def __get_type(self):
        return self.ident.split(os.sep)[0]

    type = property(__get_type, None, None,
            """ Tipo de fichero (data, static_conf, etc)
            """)


    def sum_up (self, verbose = False):

        """ Devuelve una cadena resumiendo C{este} objeto. Si C{verbose} es
            C{True}, entonces se da toda la información, no solo directorio y
            nombre de archivo. """

        #result = str (self.directory) + '/' + str (self.filename)
        result = self.ident

        if verbose:
            result = result + ' (' + str (self.checksum) + ') '

            if not self.modified:
                result = result + 'not '

            result = result + 'modified on ' + str (self.modifDate)

        return result



class FileData (FileMetaData):

    """ Datos de un archivo, incluyendo el propio contenido del archivo. """

    def __init__ (self, basepath, relpath):
        FileMetaData.__init__(self, basepath, relpath)
        path = basepath + os.sep + relpath
        self.content = open(path).read()
        self.diff = '[Aquí iría la diferencia entre las dos versiones de este archivo.]'


    def new_from_meta (meta):
        ''' Crear y devolver un objeto L{FileData} que se correponda con otro objeto
            L{FileMetaData}. '''
        result = FileData(meta.directory, meta.filename)
        return result

    # La siguiente línea convierte al método anterior en método de clase:
    new_from_meta = staticmethod (new_from_meta)


    def write(self, centre = None):
        """
        Guarda los datos del fichero en disco.

        Si se le pasa un centro, se supone q los datos se están guardando en el
        servidor. Si no se le pasa, los datos se guardan desde el cliente.
        """

        logger = logging.getLogger()
        if centre:
            # In Server
            basepath = centre.path + 'info'
            filepath = basepath + os.sep + self.filename
            self.path = (basepath, self.filename)
            checksum_file = centre.checksum_file
            modcache_file = centre.modcache_file
        else:
            # In client
            from client.daemon.config import DATA_DIR, CHECKSUM_FILE
            from client.daemon.config import MOD_FLAG_FILE
            filepath = DATA_DIR + os.sep + self.ident
            self.path = (DATA_DIR, self.ident)
            checksum_file = CHECKSUM_FILE
            modcache_file = MOD_FLAG_FILE

        # Update Checksum
        logger.debug(checksum_file)
        checksum_cache = ChecksumCache(checksum_file)
        checksum_cache[self.ident] = self.checksum

        # Update ModCache
        mc = ModCache(modcache_file)
        try:
            del mc[self.ident]
        except:
            pass

        logger.debug("set_data/filepath: %s" % filepath)
        self.__save(filepath)

    def __save(self,  filepath):
        """ Este método guarda el fichero en el path selecionado.

        Este path puede contener distintos niveles de directorios, se intentará
        crear todos y cada uno de ellos.
        """
        dirpath = os.sep.join(filepath.split(os.sep)[:-1])
        try:
            os.makedirs(dirpath)
        except:
            pass

        fileobj = open(filepath, 'w')
        fileobj.write(self.content)
        fileobj.close()




class FileMetaDataList(list):

    """ Listado de metadatos de archivos. """

    def __init__ (self, basepath = None):

        """ Crear un listado a partir de la información en el equipo local.
            Si se especifica un C{centre}, entonces es que estamos en el
            servidor, y se creará el listado correspondiente a ese centro.
            En otro caso, se asume que estamos en un cliente. Si C{debug}
            B{no} es C{None}, entonces se espera una lista de enteros, y se
            creará un listado acorde, para usarlo para depuración. """

        list.__init__(self)

        if basepath:
            self.basepath = os.sep + basepath.strip(os.sep)
            # basepath apunta al directorio info (/var/vaguada/info en los clientes,
            # /var/vaguada/12/14/info en el servidor
            self.__scan_for_files()


    def get_file(self, filename):
        for filemd in self:
            if filemd.filename == filename:
                return filemd
        raise KeyError, "Filename isn't in list"


    def get_missed_files (self, other):

        """ Calcula qué archivos de C{other} faltan en C{este} objeto.
            En abstracto, podemos pensar que el resultado es una
            diferencia de conjuntos:
            C{other - este} """

        result = FileMetaDataList()

        filename_list = [ele.filename for ele in self]
        corrupted_files = other.get_corrupted_files()

        for otherfmd in other:
            if not otherfmd in corrupted_files:

                # File is not in self
                if not otherfmd.filename in filename_list:
                    result.append(otherfmd)
                    continue

                thisfmd = self.get_file(otherfmd.filename)
                # Files cheksum not equal
                if otherfmd.checksum != thisfmd.checksum:

                    if thisfmd.corrupted and (not otherfmd.corrupted):
                        result.append(otherfmd)
                        continue

                    if (not thisfmd.corrupted) and (not otherfmd.corrupted):
                        if thisfmd.modified and otherfmd.modified:
                            if otherfmd.type == 'static_conf':
                                result.append(otherfmd)
                            # New incidence
                            continue
                        
                        elif otherfmd.modified:
                            result.append(otherfmd)
                            continue

        return result

    def get_corrupted_files (self):
        ''' Devuelve otro objeto L{FileMetaDataList} con el conjunto de
            archivos que, habiendo cambiado, o habiendo sido añadidos,
            dan lugar a una incidencia.
            Por ejemplo, porque un archivo estático de configuración ha
            cambiado.
            [TODO]'''

        return FileMetaDataList()


    def incorporate (self, other):

        """ Añade a C{este} objeto los archivos de C{other}. """

        for i in other.files:
            found = False

            for j in self.files:
                if i.directory == j.directory and \
                   i.filename == j.filename and \
                   i.checksum == j.checksum:
                    found = True
                    break

            if not found:
                self.files.add (i)

    def sum_up (self, verbose = False):

        """ Devuelve una lista de cadenas resumiendo el listado de
            archivos que C{este} objeto contiene. Si C{verbose} es
            C{True}, entonces se da toda la información, no solo
            directorio y nombre de archivo. """

        result = list ()

        for i in self:
            result.append (i.sum_up (verbose))

        result.sort ()

        return result


    def __scan_dir(self, dir):
        for filename in os.listdir(self.basepath + os.sep + dir):
            filerelpath = dir + os.sep + filename
            filepath = self.basepath + os.sep + filerelpath

            if os.path.isfile(filepath):
                newfmd = FileMetaData(self.basepath, filerelpath)
                self.append(newfmd)
            elif os.path.isdir(filepath):
                self.__scan_dir(filerelpath)


    def __scan_for_files(self):
        valid_dirs = ['data', 'static_conf', 'dynamic_conf']
        for dir in os.listdir(self.basepath):
            if os.path.isdir(self.basepath + os.sep + dir):
                if dir in valid_dirs:
                    self.__scan_dir(dir)




class ChecksumCache(dict):
    """ Caché de checksums

    Esta clase representa un diccionario persistente cuyas claves son nombres de
    ficheros y cuyos valores son sus sumas de comprobación.
    """

    def __init__(self, path):
        dict.__init__(self)
        self.path = path

        if not os.path.exists(self.path):
            open(self.path, 'w').close()
        else:
            for line in open(self.path).readlines():
                try:
                    (item, checksum) = line.split()[:2]
                except:
                    (item, checksum) = (line.split()[0], '')
                dict.__setitem__(self, item, checksum)

    def __getitem__(self, key):
        return dict.__getitem__(self, key)

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self.__save()

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self.__save()

    def __save(self):
        mod_file = open(self.path, 'w')
        for key in self.keys():
            mod_file.write(key + '\t' + self[key] + '\n')
        mod_file.close()
        


class ModCache(Set):
    """ Caché de modificaciones.

    Esta clase es, en abstracto, una lista de ficheros modificados
    voluntariamente por VAGUADA.
    """

    def __init__(self, mod_flag_file):
        """
        @type mod_flag_file: string
        @param mod_flag_file: Path del fichero donde se encuentra el listado de
        ficheros modificados.
        """
        Set.__init__(self)
        self.path = mod_flag_file

        if not os.path.exists(self.path):
            open(self.path, 'w').close()
        else:
            for line in open(self.path).readlines():
                Set.add(self, line.strip())


    def add(self, value):
        Set.add(self, value)
        self.__save()


    def remove(self, key):
        Set.remove(self, key)
        self.__save()


    def __save(self):
        mod_file = open(self.path, 'w')
        data_list = list(self)
        data_list.sort()
        for ele in data_list:
            mod_file.write(ele + '\n')
        mod_file.close()

# Para hacer pruebas de estas clases:

if '__main__' == __name__ :

    fmdl = FileMetaDataList('/var/vaguada/info/')

    for ele in fmdl:
        print "Base: %s, RPath: %s, Ident: %s" %\
                (ele.directory, ele.filename, ele.ident)
# Fin del archivo.
