#!/usr/bin/python
# -*- coding: utf-8 -*-

""" Clases base para todas las incidencias que se pueden dar en el sistema.

Toda incidencia debe tener un método para ser resuelta en el servidor y/o en el
cliente.
"""

class IncidenceData(object):
    """ Representa los datos de una incidencia.

    Las instancias de esta clase son usadas para transmitir los datos de una 
    incidencia entre el cliente y el servidor.
    """
    def __init__(self, ident, name = '', description = ''):
        """
        @type name: str
        @param name: Nombre o descripción corta de la incidencia.

        @type description: str
        @param description: Descripción larga de la incidencia.
        """

        self.__ident = ident
        self.__name = name 
        self.__description = description
        self.__methods = []
        self.__type = None
        self.__manual_solve_desc = ''
        self.__details = ''

    def __eq__(self, other):
        return  self.__class__ == other.__class__ and \
                self.name == other.name and \
                self.description == other.description 


    def __ne__(self, other):
        return not self == other

    def __str__(self):
        result = "\n\tID: %s\n\tNombre: %s\n\tDescripción: %s\n" %\
                       (self.ident, 
                        self.name,
                        self.description)
        return result


    # ATTRIBUTES ####################################################

    # ident
    def __get_ident(self):
        return self.__ident

    ident = property(__get_ident, None, None,
            """ Identificador único de la incidencia

            @type: str
            """)

    # name
    def __get_name(self):
        return self.__name

    name = property(__get_name, None, None,
            """ Nombre o descripción corta de la incidencia.

            @type: str
            """)

    # description
    def __get_description(self):
        return self.__description

    def __set_description(self, value):
        self.__description = value

    description = property(__get_description, __set_description, None,
            """ Descripción larga de la incidencia.

            @type: str
            """)

    # methods
    def __get_methods(self):
        return self.__methods

    def __set_methods(self, value):
        self.__methods = value

    methods = property(__get_methods, __set_methods, None,
            """ Lista que contiene información sobre los métodos por los que se
            puede resolver la incidencia.

            Características:

                - Cada elemento de esta lista es una cadena que describe el 
                método a utilizar. 
                - La posición de un elemento en la lista es importante, ya
                que será utilizada para seleccionar el método a utilizar para
                reparar la incidencia.

            @type: list
            """)

    # manual_solve_desc
    def __get_msd(self):
        return  self.__manual_solve_desc

    def __set_msd(self, value):
        self.__manual_solve_desc = value

    manual_solve_desc = property(__get_msd, __set_msd, None,
            """ Descripción de la resolución manual.

            Indica lo que se ha hecho para solucionar la incidencia cuando ésta
            se resuelve de forma manual.
            """)

    # type
    def __get_type(self):
        return self.__type

    def __set_type(self, value):
        self.__type = value

    type = property(__get_type, __set_type, None,
            """ Tipo de incidencia.

            Cada incidencia pertenecerá a un determinado tipo. Esto permite que
            desde la interfaz de usuario se pueda mostrar, por ejemplo, un icono
            representativo de cada tipo de incidencia.

            Existirá prácticamente un tipo de incidencia por cada clase que
            represente una incidencia. La diferenciación de cada incidencia se
            podría hacer comprobando si la incidencia que hay que mostrar
            pertenece a una determinada clase de incidencia, pero se ha escogido
            hacerlo a través de un atributo para que 1)el servidor no tenga
            constancia de ninguna clase perteneciente al cliente y 2) para que
            el tipo se pueda transmitir fácilmente a través de la red
            (recordemos que lo que se transmite a todos los efectos es un objeto
            de tipo L{IncidenceData})

            @type: str
            """)


    def __get_details(self):
        return self.__details

    def __set_details(self, value):
        self.__details = value

    details = property(__get_details, __set_details, None,
            """ Detalles de la incidencia

            Texto con formato (tabuladores y demás morralla), de gran tamaño.
            Este texto es una extensión de la descripción de la incidencia
            """)
        


class Incidence(IncidenceData):
    """ Clase abstracta que representa una incidencia.

    Todas las incidencias, tanto del servidor como del cliente, deben implemetar
    todos los métodos de esta clase.
    """
    def __init__(self, ident, name, description):
        IncidenceData.__init__(self, ident, name, description)


    def solve(self, method_pos, *params):
        """ Resolver la incidencia por el método indicado.

        @type method_pos: int
        @param method_pos: Número del método utilizado para resolver la incidencia.
        Este número es el orden que ocupa. El  método de resolución manual
        siempre debe ocupar el primer lugar.
        """
        raise NotImplementedError


    def is_active(self):
        """ Método usado para comprobar si una determinada incidencia se sigue
        dando.

        @rtype: boolean
        @return: True si la incidencia está activa (se sigue dando), False en 
        caso contrario.
        """
        return False


    def discard(seld):
        """ Método para dar de baja una incidencia que ha perdido su validez.
        """
        raise NotImplementedError
