#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Definición de las interfaces necesarias 
para la comunicación con el cliente (centro Guadalinfo)
"""

from common.interfaces.base import IUIAssistant

class IClient:
    """
    Interfaz que sirve de "puerta de entrada" a las demás interfaces exportadas
    por un cliente (centro Guadalinfo)
    """

    def __init__(self):
        raise NotImplementedError

    
    def get_ui_assistant(self):
        """
        @rtype: L{IUIAssistant}
        @return: implementación de la interfaz IUIAssistant para la
        comunicación de datos con  la interfaz de usuario del cliente.
        """
        raise NotImplementedError


    def get_data_manager(self):
        """
        @rtype: L{IClientDataManager}
        @return: implementación de la interfaz IClientDataManager para la
        obtención de datos por parte del servidor sobre el cliente.
        """
        raise NotImplementedError


    def get_incidence_manager(self):
        """
        @rtype: L{IClientIncidenceManager}
        @return: implementación de la interfaz IClientIncidenceManager para la
        gestión de incidencias por parte del servidor.
        """
        raise NotImplementedError


class IClientDataManager:
    """
    Interfaz que se usa para obtener todo tipo de datos de un centro guadalinfo. 

    Es la puerta de entrada a los centros. El principal de usuario de  esta
    interfaz es el demonio del servidor, el cual utiliza esta interfaz para
    obtener el estado de los envíos realizados por los centros.
    """

    def __init__(self):
        raise NotImplementedError


    def ping(self):
        """
        Método utilizado para saber si un centro está conectado.

        @rtype: boolean
        @return: C{True} si el centro está conectado
        """
        raise NotImplementedError


    def set_dispatch_list(self, dispatch_list):
        """
        Método utilizado por el servidor para indicar al cliente su deseo de
        sincronizar, informándole de los envíos que el servidor contiene, para
        que el cliente envíe lo que falta o pida lo que sobra.

        @type dispatch_list: Lista de objetos FileMetaData 
        @param dispatch_list: Lista que contiene los identificadores de todos
        los envíos que se suponen realizados por el  centro. La implementación
        de IClientDataManager se encargará de comparar esa lista con la suya
        propia para enviar información que no esté en el servidor o incluso para
        pedir información que por cualquier motivo falta en el cliente.

        @rtype: FileMetaDataList tuple
        @return: Tupla en la que el primer elemento es una lista con los
        ficheros que están en el servidor y no en el cliente y el segundo, una
        lista con los ficheros que están en el cliente y no en el servidor.
        """
        raise NotImplementedError


    def get_datetime(self):
        """
        @return: Hora local del cliente.
        """
        raise NotImplementedError


    def set_code(self, new_code):
        """ Establece el código del centro

        @type new_code: tuple
        @param new_code: Nuevo código del centro
        """
        raise NotImplementedError
        




class IClientIncidenceManager:
    """
    Interfaz encargada de gestionar las incidencias ocurridas en un centro.

    Sus funcionalidades se agrupan en:
        - Obtención de un listado de incidencias.
        - Solución de incidencias.
    """
    def __init__(self):
        raise NotImplementedError


    def get_ninc(self):
        """ Obtiene el número de incidencias actuales del centro
        """
        raise NotImplementedError

    
    def get_incidences(self):
        """
        Método que sirve para obtener un listado de incidencias.
        
        @rtype: ({}, {}, ...)
        @return: Listado de incidencias ocurridas en el centro. El formato del
        tipo devuelto es equivalente al especificado en
        L{IUIAssistant.get_incidences}
        """
        raise NotImplementedError


    def is_active(self, incidence_data):
        """ Comprueba si una incidencia sigue vigente o no en el sistema

        @type incidence_data: L{IncidenceData}
        @param incidence_data: datos de la incidencia. Lo que realmente importa
        aquí es el identificador de la incidencia

        @rtype: boolean
        @return: True si la incidencia sigue activa, False en otro caso.
        """
        raise NotImplementedError


    def solve(self, incidence_data, method_pos, *params):
        """ Resuelve una incidencia utilizando el método indicado

        @type incidence_data: IncidenceData
        @param incidence_data: Datos de la incidencia que se quiere reparar.
        """
        raise NotImplementedError


class IClientUIAssistant(IUIAssistant):

    def get_incidences(self):
        raise NotImplementedError
