#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Definición de las interfaces necesarias para la
comunicación con el servidor, tanto por parte de los clientes como parte de la
interfaz gráfica de usuario.
"""

from base import IUIAssistant
from common.incidences import IncidenceData #@UnusedImport


class IServerDataManager:
    """
    Esta interfaz es responsable de la transmisión de datos entre los clientes y
    el servidor. 

    Sus principales funcionalidades deben ser:

        - Dar soporte al envío de datos desde los clientes.
        - Disponer de un mecanismo de obtención de datos ya enviados que cada
          cliente, para que cada cliente pueda conocer en todo momente los
          envíos realizados.
    """
    def __init__(self):
        raise NotImplementedError


    def ping(self, centre_code): #@UnusedVariable
        """Efectúa un ping al servidor. 

        Devuelve True si la conexión es satisfactoria.
        """
        return True


    def start_sync(self, center_code):
        """
        Método utilizado por los clientes para iniciar una sincronización.

        En la implementación de este método se llamará a métodos del cliente, y
        éste a su vez, llamará a métodos del servidor.

        @type center_code: tuple.
        @param center_code: Tupla que identifica a un centro. Con este centro es
        con el que se iniciará la transferencia, siempre que la dirección ip del
        que llama al método coincida con la dirección ip asociada al código del
        centro. Si no coincide, se creará una incidencia de resolución
        automática.
        """
        raise NotImplementedError


    def end_sync(self, center_code):
        """
        Utilizado por los clientes para finalizar un sincronización.

        @type center_code: tuple.
        @param center_code: Tupla que identifica a un centro. Con este centro es
        con el que se iniciará la transferencia, siempre que la dirección ip del
        que llama al método coincida con la dirección ip asociada al código del
        centro. Si no coincide, se creará una incidencia de resolución
        automática.
        """
        raise NotImplementedError


    def get_dispatch_list(self, centre_code):
        """ Utilizado por los clientes para obtener un listado con los envíos
        realizados.

        @type centre_code: tuple
        @param centre_code: Código del centro
        """
        raise NotImplementedError


    def set_data(self, center_code, file_data):
        """
        Método utilizado por el cliente para enviar datos hasta el servidor.

        @type center_code: tuple
        @param center_code: Código del centro que quiere enviar los datos.

        @type file_data: Instancia o lista de instancias de FileData.
        @param file_data: Contenido a transmitir dirección cliente-servidor.
        """
        raise NotImplementedError


    def get_data(self, center_code, file_meta_data):
        """
        Método utilizado por el cliente para obtener el contenido de un
        determinado envío.

        @type center_code: tuple
        @param center_code: Código del centro que quiere enviar los datos.

        @type file_meta_data: instancia de FileMetaData 
        @param file_meta_data: Metadatos del envío que se quiere transmitir
        dirección servidor-cliente.

        @rtype: FileData 
        @return: Fichero de envío que se transmite.
        """
        raise NotImplementedError


    def get_time(self):
        """
        @return: Hora local del servidor.
        """
        raise NotImplementedError


    def log_inc(self, centre_code, msg):
        """ Escribe un mensaje en el log de incidencias del centro indicado.
        """ 
        raise NotImplementedError


    def log_sync(self, centre_code, msg):
        """ Escribe un mensaje en el log de envíos del centro indicado.
        """ 
        raise NotImplementedError


    def get_inc_log(self, centre_code):
        """ Obtiene el contenido del log de incidencias del centro indicado.
        """
        raise NotImplementedError


    def get_sync_log(self, centre_code):
        """ Obtiene el contenido del log de envíos del centro indicado.
        """
        raise NotImplementedError


    def get_last_dispatch_date(self, centre_code):
        """ Devuelve la fecha del último envío realizado por el centro.
        """
        raise NotImplementedError


    def get_package_list(self):
        """ 
        @rtype: list
        @return: Lista de tuplas con información sobre los paquetes que
        se deben tener instalados en el sistema
        """
        raise NotImplementedError


    def get_package_list_checksum(self):
        """
        Devuelve la suma md5 de la lista de paquetes
        """
        raise NotImplementedError


    def get_static_conf_list(self):
        """
        @rtype: list
        @return: Lista de rutas de ficheros de configuración que pertenecen a la
        categoría de "estática".
        """
        raise NotImplementedError


    def get_static_conf_list_checksum(self):
        """
        @rtype: str
        @return: Suma md5 de la lista de ficheros de configuración estática
        """
        raise NotImplementedError

    
    def set_version(self, centre_code, content):
        """ Guarda el contenido sobre la versión del sistema operativo instalado
        en el cliente en el fichero /var/vaguada/ / /lsb-release
        """
        raise NotImplementedError


class IServerUIAssistant(IUIAssistant):
    """
    Interfaz que exporta los métodos necesarios para la interfaz de usuario del
    servidor.
    """
    def __init__(self):
        raise NotImplementedError


    def get_all_centre_data(self):
        """
        Método utilizado para obtener un listado con información resumen de
        todos los centros.

        @rtype: Diccionario de diccionarios
        @return: Diccionario con datos sobre todos los centros, a modo de
        resumen para poder ofrecer al usuario un listado del estado global de
        los centros. Las claves son los identificadores de los centros y los
        valores, diccionarios con las propiedades de cada centro: 
        Las propiedades de cada centro son:

            - "ninc": int. Número de incidencias actuales de ese centro.
            - "conn": bool. Estado de la conexión: True para conectado, False para
            desconectado.

        Un ejemplo::

            {
                (2,14):{
                    "ninc": 2,
                    "conn": True
                }

            }
        """
        raise NotImplementedError


    # getters and setters for config files
    def get_packages(self):
        """ Devuelve el contenido del fichero con  el  listado de paquetes
        """
        raise NotImplementedError


    def set_packages(self, content):
        """ Guarda el texto indicado con content en el fichero con el listaod
        de paquetes
        """
        raise NotImplementedError


    def get_files(self):
        """ Devuelve el contenido del listado de ficheros a los que se le debe
        hacer copia de seguridad
        """
        raise NotImplementedError


    def set_files(self, content):
        """ Guarda el texto indicado con L{content} en el fichhero con el
        listaod de ficheros a los que hacer copia de seguridad
        """
        raise NotImplementedError


    def get_centres(self):
        """ Devuelve el contenido del fichero con los datos de los centros dados
        de alta
        """
        raise NotImplementedError


    def set_centres(self, content):
        """ Guarda el texto indicado con L{content} en el fichero con los datos
        de los centros dados de alta
        """
        raise NotImplementedError


    def start_ssh(self, centre_code):
        """ Prepara el centro para realizar conexiones por ssh.

        @rtype: str
        @return: Dirección del centro
        """
        raise NotImplementedError

    
    def end_ssh(self, centre_code):
        """ Bloquea las conexiones por ssh en el centro
        """
        raise NotImplementedError


    def get_role(self):
        """ Método encargado de asignar un rol a una interfaz de usuario
        dependiendo de la dirección ip desde donde se conecte.
        """
        raise NotImplementedError



class IServerIncidenceManager:
    """ Interfaz que representa al gestor de incidencias del servidor en remoto.

    Contiene los métodos necesarios para que los centros puedan obtener el
    listado de incidencias presentes en el servidor y poder gestionarlos (dar de
    alta/baja a incidencias, etc).
    """

    #def get_incidences(self, centre_code):
    #    """ Obtiene todas las incidencias pertenecientes a un determinado centro
    #    de las que se tiene constancia.

    #    @type centre_code: tuple
    #    @param centre_code: Código del centro del cual se quiere obtener el
    #    listado de incidencias.

    #    @rtype: IncidenceData list
    #    @return: Lista de incidencias perteneciente al centro indicado.
    #    """
    #    raise NotImplementedError


    def set_ninc(self, centre_code, ninc):
        """ Informa al servidor que un determinado centro tiene ninc
        incidencias.
        """
        raise NotImplementedError


    def add(self, centre_code, incidence_type, params = ()):
        """ Añade una incidencia a la lista de incidencias a la lista de
        incidencias de servidor de un centro determinado.

        @type centre_code: tuple
        @param centre_code: Código del centro a cuya lista se va a añadir la
        incidencia.

        @type incidence_type: str
        @param incidence_type: Tipo de la incidencia

        @type params: tuple
        @param params:  Parámetros  a pasar al constructor de la incidencia.
        """
        raise NotImplementedError


    def remove(self, centre_code, incidence_data):
        """ Borra una incidencia de la lista de incidencias perteneciente al
        centro indicado.
        """
        raise NotImplementedError

