#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Utilidades para el servidor y el cliente 

En este módulo se encuentran algunas clases y funciones (decoradores en
realidad) que sirven de utilidad en diversos puntos del servidor.

Todas las excepciones comunes a cliente y  servidor también tienen su cabida en
este módulo.
"""

import os
import os.path
import pickle
import datetime
import Pyro.errors
import Pyro.core


class CentreCodeError(Exception):
    """Excepción que se lanza al comprobar que un centro no existe en el
    sistema.
    """
    pass


class GUINotAuthorized (Exception):
    """ Excepción que indica que una interfaz de usuario no está autorizada para
    ejecutar un determinado método remoto
    """
    pass


def read_text_file (file_name):

    ''' Cargar el contenido de un archivo de texto.
        @param file_name: la ruta a un archivo de texto.
        @type file_name: L{str}
        @return: el contenido del archivo, o L{None} si hay errores.
        @rtype: L{str} '''

    try:
        tmpFile = file (file_name, 'r')
        result = tmpFile.read ()
        tmpFile.close ()
    except:
        result = None

    return result

def write_text_file (file_name, text):

    ''' Volcar texto en un archivo.
        @param file_name: la ruta a un archivo.
        @type file_name: L{str}
        @param text: el texto a escribir.
        @type text: L{str}
        @return: si ha habido éxito, o no.
        @rtype: L{bool} '''

    try:
        tmpFile = file (file_name, 'w')
        tmpFile.write (text)
        tmpFile.close ()
        result = True
    except:
        result = False

    return result


class PersistentDict(dict):
    """ Diccionario persistente usando pickle para la serialización
    """

    def __init__(self, filepath):
        dict.__init__(self)
        self.filepath = filepath

        if not os.path.exists(filepath):
            self.__save()
        else:
            fileobj = open(filepath, 'r')
            datadict = pickle.load(fileobj)
            fileobj.close()
            self.update(datadict)

    def __setitem__(self, newkey, newvalue):
        dict.__setitem__(self, newkey, newvalue)
        self.__save()


    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self.__save()

    def __save(self):
        fileobj = open(self.filepath, 'w')
        pickle.dump(self, fileobj)
        fileobj.close()



def get_last_file_date(path, extension = '.users'):
    """
    @rtype: tuple
    @return: Tupla que contiene: en el primer elemento la fecha (datetime.date)
    del último fichero encontrado, y en el segundo, su contenido.
    """
   
    file_list = []
    maxdate = '0'
    for filepath in os.listdir(path):
        if filepath.endswith(extension):
            file_date = filepath.split('.')[0]
            if file_date > maxdate:
                maxdate = file_date
    if maxdate != '0':
        max_content = open(path + os.sep + maxdate + extension).read()
        try:
            date = datetime.date(int(maxdate[:4]), int(maxdate[4:6]), int(maxdate[6:8]))
            return (date, max_content)
        except:
            return (None, None)
    else:
        return (None, None)


def ui_connection_safe(method):

    def new_method (self, *args, **kargs):
        import gtk
        result = None
        try:
            result = method(self, *args, **kargs)

        except (Pyro.core.ProtocolError, 
                Pyro.core.ConnectionClosedError,
                Pyro.errors.ProtocolError, 
                Pyro.errors.ConnectionClosedError,

                Pyro.errors.URIError):

            msg = 'No se puede conectar con el demonio.'
            dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR,
                buttons = gtk.BUTTONS_CLOSE,
                message_format = msg)

            gtk.threads_enter()
            dialog.run()
            dialog.destroy()
            gtk.threads_leave()

        return result


    # Set new method doc from old method (for epydoc)
    new_method.__doc__ = method.__doc__
    return new_method
# Fin del archivo.

